--[[
Copyright (c) 2017 raidho36/rcoaxil

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
]]--

local mmax, mmin, mceil, mfloor, mhuge = math.max, math.min, math.ceil, math.floor, math.huge

local Util = { }

function Util.clamp ( x, max, min )
	return mmax ( mmin ( x, max ), min )
end

function Util.lerp ( t, a, b )
	return a + ( b - a ) * t
end

function Util.map ( t, a, b, x, y )
	return x + ( y - x ) * ( ( t - a ) / ( b - a ) )
end

function Util.diff ( a, b, v )
	return ( a - b + v ) % ( v + v ) - v
end

function Util.sign ( a )
	return mmin ( 1, mmax ( -1, mceil ( a ) + mfloor ( a ) ) )
end

function Util.perftest ( name, func, count, iters )
	io.write ( "testing " .. name .. " " )
	io.flush ( )
	local timing = { }
	local check = 0
	for c = 1, count do
		local t = love.timer.getTime ( )
		for i = 1, iters do check = check + func ( ) end
		timing[ #timing + 1 ] = love.timer.getTime ( ) - t
		io.write ( "." )
		io.flush ( )
	end

	table.sort ( timing )
	local output = { min = timing[ 1 ], max = timing[ 1 ], avg = 0, mean = 0 }
	for i = 1, #timing do
		output.min = math.min ( output.min, timing[ i ] )
		output.max = math.max ( output.max, timing[ i ] )
		output.avg = output.avg + timing[ i ] / #timing
	end
	if #timing == 1 then
		output.mean = timing[ 1 ]
	elseif #timing % 2 == 1 then
		output.mean = ( timing[ math.floor ( #timing / 2 ) ] + timing[ math.ceil ( #timing / 2 ) ] ) / 2
	else
		output.mean = timing[ #timing / 2 ]
	end
	output.name = name
	output.check = check
	print ( )
	return output
end

function Util.printperf ( data )
	if not data then
		print ( '', 'min', 'max', 'avg', 'mean', 'check' )
	else
		print ( data.name,
		math.ceil ( data.min * 10000 ) / 10000,
		math.ceil ( data.max * 10000 ) / 10000,
		math.ceil ( data.avg * 10000 ) / 10000,
		math.ceil ( data.mean * 10000 ) / 10000,
		data.check
		)
	end
end

return Util
